#pragma once

namespace Amethyst
{
  typedef unsigned int Reg;
  typedef unsigned char Byte;

  class Registers
  {
  public:
    Registers(const Registers& copy);
    Registers();

    Reg R[16];
    Reg Cpsr;

    inline bool N() { return GetCpsrBit(NBit); }
    inline bool Z() { return GetCpsrBit(ZBit); }
    inline bool C() { return GetCpsrBit(CBit); }
    inline bool V() { return GetCpsrBit(VBit); }

    inline void SetN(bool value) { SetCpsrBit(value, NBit); }
    inline void SetZ(bool value) { SetCpsrBit(value, ZBit); }
    inline void SetC(bool value) { SetCpsrBit(value, CBit); }
    inline void SetV(bool value) { SetCpsrBit(value, VBit); }

    inline static bool Bit(Reg value, int shift) { return (value & (1 << shift)) != 0; }
    inline static Reg Bits(Reg value, int start, int end) { return (value >> start) & (0xffffffff >> (31 - (end - start))); }

  private:
    inline bool GetCpsrBit(int flags)
    {
      return (Cpsr & flags) != 0;
    }

    inline void SetCpsrBit(bool value, int flags)
    {
      if (value)
      {
        Cpsr |= flags;
      }
      else
      {
        Cpsr &= flags ^ 0xffffffff;
      }
    }

    const int NBit = (1 << 31);
    const int ZBit = (1 << 30);
    const int CBit = (1 << 29);
    const int VBit = (1 << 28);
  };

}
